import tellurium as te
import matplotlib.pyplot as plt

def reactionAntimony(dict, currentK):
    #f = open("webapp/static/antimony1.txt", 'a')
    f = open("AMCowmvg/static/antimony1.txt", 'a')

    reactantSpecies = dict['Reactants']
    FixedReactants = dict['FixedReactants']
    newStr = ''
    for species in reactantSpecies:
        for fixedSpecies in FixedReactants:
            if species == fixedSpecies:
                species = '$' + species
                newStr = newStr + species + '+'
            else:
                newStr = newStr + species + '+'
    newStr = newStr[:-1] + '->'

    productSpecies = dict['Products']
    FixedProducts = dict['FixedProducts']
    for species in productSpecies:
        for fixedSpecies in FixedProducts:
            if species == fixedSpecies:
                species = '$' + fixedSpecies
                newStr = newStr + species + '+'
            else:
                newStr = newStr + species + '+'

    rxnVals = dict['RxnConstant']
    kVal = len(rxnVals)
    kList = []

    for i in range(kVal):
        numStr = str(i + currentK)
        kList.append('k' + numStr)
        numConstants = i

    currentK = currentK + kVal
    newStr = newStr[:-1] + ' ; ' + kList[0] + '*'

    for species in reactantSpecies:
        newStr = newStr + species + '*'
    newStr = newStr[:-1]

    # print this line to the antimony file
    f.write('\n')
    f.write(newStr)

    if dict['Reversibility'] == True:
        secStr = ''
        for species in productSpecies:
            for fixedSpecies in FixedReactants:
                if species == fixedSpecies:
                    species = '$' + fixedSpecies
                    secStr = secStr + species + '+'
                else:
                    secStr = secStr + species + '+'
        secStr = secStr[:-1] + '->'
        for species in reactantSpecies:
            for fixedSpecies in FixedReactants:
                if species == fixedSpecies:
                    species = '$' + species
                    secStr = secStr + species + '+'
                else:
                    secStr = secStr + species + '+'

        secStr = secStr[:-1] + ' ; ' + kList[1] + '*'

        for species in productSpecies:
            secStr = secStr + species + '*'
        secStr = secStr[:-1]

        # print secStr to antimony file within the if statement
        f.write('\n')
        f.write(secStr)
    f.close()
    return currentK, kList

def conditionsAntimony(dict,  kList):
    #f = open('webapp/static/antimony2.txt','a')
    f = open('AMCowmvg/static/antimony2.txt','a')
    reactantIC = dict['ReactantIC']
    reactantSpecies = dict['Reactants']
    str1 = ''
    count = 0
    for species in reactantSpecies:
        str1 += species + '=' + reactantIC[count] + ';'
        count += 1
    str1 = str1[:-1]
    f.write('\n')
    f.write(str1)
    # print to IC file

    productIC = dict['ProductIC']
    productSpecies = dict['Products']
    str2 = ''
    count = 0
    for species in productSpecies:
        str2 += species + '=' + productIC[count] + ';'
        count += 1
    str2 = str2[:-1]
    # print to IC file
    f.write('\n')
    f.write(str2)

    rxnConstants = dict['RxnConstant']
    str3 = ''
    count = 0
    for var in kList:
        str3 += var + '=' + rxnConstants[count] + ';'
        count += 1
    str3 = str3[:-1]
    f.write('\n')
    f.write(str3)
    f.close()

    return str1, str2, str3

def resetFiles():
    #with open('webapp/static/antimony1.txt', 'r+') as file:
    with open('AMCowmvg/static/antimony1.txt', 'r+') as file: 
        file.truncate(0)
        file.close()
    #with open('webapp/static/antimony2.txt', 'r+') as file:
    with open('AMCowmvg/static/antimony2.txt', 'r+') as file:
        file.truncate(0)
        file.close() 

"""
Takes in given values as strings or booleans from the website program and saves
them to a dictionary
Inputs:
    - reactants: (Str) comma separated string of a list of letters that represent reactants
    - fixed_reactants: (Str) comma separated string of the reactants that are fixed values
    - reactantIC: (Str) comma separated string of numbers that are the IC of the reactants
    - products: (Str) comma separated string of a list of letters that represent products
    - fixed_products: (Str) comma separated string of the products that are fixed values
    - productIC: (Str) comma separated string of numbers that are the IC of the products
    - reactionConstant: (Str) comma separates string of numbers that are the k values of reaction
    - reversibility: (boolean) true if the reaction is reversible
Outputs:
    - dict: a dictionary that contains the values as the inputs connected to a name
"""
def init(reactants, fixed_reactants, reactantIC, products, fixed_products, productIC, reactionConstant, reversibility):
    dict = {}

    dict['Reactants'] = reactants.split(',')

    dict['FixedReactants'] = fixed_reactants.split(',')

    dict['ReactantIC'] = reactantIC.split(',')

    dict['Products'] = products.split(',')

    dict['FixedProducts'] = fixed_products.split(',')

    dict['ProductIC'] = productIC.split(',')

    dict['RxnConstant'] = reactionConstant.split(',')

    dict['Reversibility'] = reversibility

    return dict

def runSim():
    #file1 = open('webapp/static/antimony1.txt','a')
    #file2 = open('webapp/static/antimony2.txt','r')

    file1 = open('AMCowmvg/static/antimony1.txt','a')
    file2 = open('AMCowmvg/static/antimony2.txt','r')

    for line in file2:
        file1.write(line)
        print(line)
    file1.close()
    file2.close()

    loadModel()

def loadModel():
    #r = te.loadAntimonyModel('webapp/static/antimony1.txt')
    r = te.loadAntimonyModel('AMCowmvg/static/antimony1.txt')

    result = r.simulate()
    r.plot()
    #plt.savefig("webapp/static/output.jpg")
    plt.savefig("AMCowmvg/static/output.jpg")

def loadSBML(url):
    r = te.loadSBMLModel(url)
    result = r.simulate()
    r.plot()
    #plt.savefig("webapp/static/output.jpg")
    plt.savefig("AMCowmvg/static/output.jpg")


from flask import render_template, redirect, url_for
#from AMCowmvg.antimonyTools import *
from antimonyTools import *
#from AMCowmvg.forms import addReaction, submit, reset, upload, SBML
from forms import addReaction, submit, reset, upload, SBML
#from AMCowmvg import app
from __init__ import app
import os

currentK = 0

@app.route('/')
def home():
    resetFiles()
    return redirect(url_for('build'))

@app.route('/build', methods=['GET', 'POST'])
def build():
    global currentK
    form = addReaction()
    form2 = submit()
    form3 = reset()
    form4 = upload()
    form5 = SBML()
    if form.validate() and form.submit.data:
        reactants = form.reactNames.data
        reactantIC = form.startingReactConcs.data
        fixed_reactants = form.fixedReactConc.data
        products = form.productNames.data
        productIC = form.startingProductConcs.data
        fixed_products = form.fixedProductConc.data
        rxn_constants = str(form.reactionConstant.data)
        reversibility = form.reversible.data
        dict = init(reactants, fixed_reactants,reactantIC,products,fixed_products,productIC,rxn_constants,reversibility)
        currentK, kList = reactionAntimony(dict,currentK)
        f,g,h = conditionsAntimony(dict, kList)
        return redirect(url_for('build'))
    if form2.validate() and form2.submit2.data:
        runSim()
        return redirect(url_for('results'))
    if form3.validate() and form3.submit3.data:
        currentK = 0
        resetFiles()
        #if (os.path.exists("webapp/static/output.jpg")):
        #    os.remove("webapp/static/output.jpg")
        if (os.path.exists("AMCowmvg/static/output.jpg")):
            os.remove("AMCowmvg/static/output.jpg")


        runSim()
    if form4.validate() and form4.submit4.data:
        #form4.model.data.save('webapp/static/antimony1.txt')
        form4.model.data.save('AMCowmvg/static/antimony1.txt')        
        loadModel()
        return redirect(url_for('results'))
    if form5.validate() and form5.submit5.data:
        loadSBML(form5.model.data)
        return redirect(url_for('results'))
        
        
    return render_template('buildSystem.html', title='Build Reactions', form=form, form2 = form2, form3 = form3, form4 = form4, form5 = form5)

@app.route('/results')
def results():

    return render_template('results.html', title="Results")


from flask import Flask

app = Flask(__name__)
app.config['SECRET_KEY'] = 'banana'

#from AMCowmvg import routes
#import AMCowmvg.routes

try:
    import AMCowmvg.routes
except:
    print("Local Execution Occured")

try:
    import routes
except:
    print("Online Execution Occured")